/*
 * cygwin_ntfs.c
 *
 * Copyright (c) 2002 Steven O'Brien <steven.obrien2@ntlworld.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "cygextras.h"
#include <windows.h>

#define SIZE_FSTYPE 256


/*
 * return 1 if filesystem containing filename is NTFS
 *        0 otherwise
 */

int check_ntfs(const char *filename)
{
	char win32_path[MAX_PATH];
	char fstype[SIZE_FSTYPE];

	/* get windows root dir from filename */
	cygwin_conv_to_full_win32_path(filename, win32_path);

    /* could be either drive:\ or \\server\share\ */
    if (win32_path[1] == ':') {
    	/* terminate string after first backslash */
		win32_path[3] = '\0';
    }
    else {
		int i;
		int slashcount = 0;
		int len = strlen(win32_path);

		/* arbitrarily put a backslash at end, just in case */
		if (len < MAX_PATH - 1) {
			win32_path[len] = '\\';
			win32_path[len + 1] = '\0';
			len++;
		}

    	/* terminate string after fourth backslash */
		for (i = 0; i < len; i++) {
			if (win32_path[i] == '\\') {
				if (++slashcount == 4) {
					if (i < MAX_PATH - 1) {
						win32_path[i+1] = '\0';
                    }
					break;
				}
			}
		}
    }

	if (GetVolumeInformation(win32_path, NULL, 0, NULL, NULL, NULL, fstype, SIZE_FSTYPE)) {
		return !strcmp(fstype, "NTFS");
	}
	else {
		return 0;
	}

}
